"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var isComponent_1 = require("./utils/isComponent");
var createAssignHelper_1 = require("./utils/createAssignHelper");
var isNullOrUndefined_1 = require("./utils/isNullOrUndefined");
var getName_1 = require("./utils/getName");
var getValue_1 = require("./utils/getValue");
var svgAttributes_1 = require("./utils/svgAttributes");
var isNodeNull_1 = require("./utils/isNodeNull");
var handleWhiteSpace_1 = require("./utils/handleWhiteSpace");
var NULL;
function default_1() {
    return function (context) {
        return transformSourceFile;
        function transformSourceFile(node) {
            var isInferno = false;
            node.statements.forEach(function (node) {
                if (node.kind === ts.SyntaxKind.ImportDeclaration) {
                    if (node.moduleSpecifier.text === "inferno") {
                        isInferno = true;
                    }
                }
            });
            if (!isInferno) {
                node.statements.unshift(ts.createImportDeclaration(undefined, undefined, ts.createImportClause(undefined, ts.createNamedImports([
                    ts.createImportSpecifier(undefined, ts.createIdentifier("createVNode"))
                ])), ts.createLiteral("inferno")));
            }
            if (node.isDeclarationFile) {
                return node;
            }
            var visited = ts.visitEachChild(node, visitor, context);
            // ts.addEmitHelpers(visited, context.readEmitHelpers());
            return visited;
        }
        function visitor(node) {
            switch (node.kind) {
                case ts.SyntaxKind.JsxElement:
                    return createVNode(node, node.children);
                case ts.SyntaxKind.JsxSelfClosingElement:
                    return createVNode(node);
                case ts.SyntaxKind.JsxText:
                    var text = handleWhiteSpace_1.default(node.getText());
                    if (text !== "") {
                        return ts.createLiteral(text);
                    }
                    break;
                case ts.SyntaxKind.JsxExpression:
                    if (node.expression) {
                        return ts.visitNode(node.expression, visitor);
                    }
                    break;
                case ts.SyntaxKind.ImportDeclaration:
                    if (node.moduleSpecifier.text === "inferno") {
                        if (node.importClause.namedBindings) {
                            var isCreateVNode_1 = false;
                            node.importClause.namedBindings.elements.forEach(function (node) {
                                if (node.name.text === "createVNode") {
                                    isCreateVNode_1 = true;
                                }
                            });
                            if (!isCreateVNode_1) {
                                node.importClause.namedBindings.elements.push(ts.createImportSpecifier(undefined, ts.createIdentifier("createVNode")));
                            }
                        }
                        else {
                            node.importClause.namedBindings = ts.createNamedImports([
                                ts.createImportSpecifier(undefined, ts.createIdentifier("createVNode"))
                            ]);
                        }
                    }
                    return node;
                default:
                    return ts.visitEachChild(node, visitor, context);
            }
        }
        function createVNode(node, children) {
            var vType;
            var vProps;
            var vChildren;
            if (children) {
                var openingElement = node.openingElement;
                vType = getVNodeType(openingElement.tagName);
                vProps = getVNodeProps(openingElement.attributes.properties, vType.isComponent);
                vChildren = getVNodeChildren(children).children;
            }
            else {
                vType = getVNodeType(node.tagName);
                vProps = getVNodeProps(node.attributes.properties, vType.isComponent);
            }
            var flags = vType.flags;
            var props = vProps.props;
            if (vProps.hasKeyedChildren) {
                flags = flags | 32 /* HasKeyedChildren */;
            }
            if (vProps.hasNonKeyedChildren) {
                flags = flags | 64 /* HasNonKeyedChildren */;
            }
            if (vType.isComponent && vChildren) {
                var addChildrenToProps = true;
                if (vChildren.kind === ts.SyntaxKind.ArrayLiteralExpression &&
                    vChildren.elements.length === 0) {
                    addChildrenToProps = false;
                }
                if (addChildrenToProps) {
                    if (props.length === 1) {
                        props[0].properties.push(ts.createPropertyAssignment(getName_1.default("children"), vChildren));
                    }
                    else {
                        props.push(ts.createObjectLiteral([
                            ts.createPropertyAssignment(ts.createIdentifier("children"), vChildren)
                        ]));
                    }
                }
                vChildren = NULL;
            }
            return ts.createCall(ts.createIdentifier("createVNode"), [], createVNodeArgs(flags, vType.type, vProps.className, vChildren, props, vProps.key, vProps.ref, vProps.noNormalize, context));
        }
        function getVNodeType(type) {
            var nodeKind = type.kind;
            var component = false;
            var flags;
            if (nodeKind === ts.SyntaxKind.Identifier) {
                if (isComponent_1.default(type.text)) {
                    component = true;
                    flags = 16 /* ComponentUnknown */;
                }
                else {
                    var tag = type.text;
                    type = ts.createLiteral(tag);
                    switch (tag) {
                        case "svg":
                            flags = 128 /* SvgElement */;
                            break;
                        case "input":
                            flags = 512 /* InputElement */;
                            break;
                        case "textarea":
                            flags = 1024 /* TextareaElement */;
                            break;
                        case "select":
                            flags = 2048 /* SelectElement */;
                            break;
                        case "media":
                            flags = 256 /* MediaElement */;
                            break;
                        default:
                            flags = 2 /* HtmlElement */;
                    }
                }
            }
            return {
                type: type,
                isComponent: component,
                flags: flags
            };
        }
        function getVNodeProps(astProps, isComponent) {
            var props = [];
            var key = null;
            var ref = null;
            var className = null;
            var hasKeyedChildren = false;
            var hasNonKeyedChildren = false;
            var noNormalize = false;
            var assignArgs = [];
            for (var i = 0; i < astProps.length; i++) {
                var astProp = astProps[i];
                if (astProp.kind === ts.SyntaxKind.JsxSpreadAttribute) {
                    assignArgs = [
                        ts.createObjectLiteral(),
                        ts.createIdentifier(astProp.expression.text)
                    ];
                }
                else {
                    var propName = astProp.name.text;
                    // if (propName.type === 'JSXIdentifier') {
                    // 	propName = propName.name;
                    // } else if (propName.type === 'JSXNamespacedName') {
                    // 	propName = propName.namespace.name + ':' + propName.name.name;
                    // }
                    if (!isComponent &&
                        (propName === "className" || propName === "class")) {
                        className = getValue_1.default(astProp.initializer);
                    }
                    else if (!isComponent && propName === "htmlFor") {
                        props.push(ts.createPropertyAssignment(getName_1.default("for"), getValue_1.default(astProp.initializer)));
                    }
                    else if (propName.substr(0, 11) === "onComponent" && isComponent) {
                        // fix
                        if (!ref) {
                            ref = ts.createObjectLiteral([]);
                        }
                        ref.properties.push(ts.createObjectLiteral(getName_1.default(propName), getValue_1.default(astProp.initializer)));
                    }
                    else if (!isComponent && propName in svgAttributes_1.default) {
                        // React compatibility for SVG Attributes
                        props.push(ts.createPropertyAssignment(getName_1.default(svgAttributes_1.default[propName]), getValue_1.default(astProp.initializer)));
                    }
                    else {
                        switch (propName) {
                            case "noNormalize":
                                noNormalize = true;
                                break;
                            case "hasNonKeyedChildren":
                                hasNonKeyedChildren = true;
                                break;
                            case "hasKeyedChildren":
                                hasKeyedChildren = true;
                                break;
                            case "ref":
                                ref = getValue_1.default(astProp.initializer);
                                break;
                            case "key":
                                key = getValue_1.default(astProp.initializer);
                                break;
                            default:
                                props.push(ts.createPropertyAssignment(getName_1.default(propName), getValue_1.default(astProp.initializer)));
                        }
                    }
                }
            }
            if (props.length)
                assignArgs.push(ts.createObjectLiteral(props));
            return {
                props: assignArgs,
                key: isNullOrUndefined_1.default(key) ? NULL : key,
                ref: isNullOrUndefined_1.default(ref) ? NULL : ref,
                hasKeyedChildren: hasKeyedChildren,
                hasNonKeyedChildren: hasNonKeyedChildren,
                noNormalize: noNormalize,
                className: isNullOrUndefined_1.default(className) ? NULL : className
            };
        }
        function getVNodeChildren(astChildren) {
            ts.SyntaxKind.NumericLiteral;
            var children = [];
            var parentCanBeKeyed = false;
            for (var i = 0; i < astChildren.length; i++) {
                var child = astChildren[i];
                var vNode = visitor(child);
                if (!isNullOrUndefined_1.default(vNode)) {
                    children.push(vNode);
                    /*
                      * Loop direct children to check if they have key property set
                      * If they do, flag parent as hasKeyedChildren to increase runtime performance of Inferno
                      * When key already found within one of its children, they must all be keyed
                      */
                    if (parentCanBeKeyed === false && child.openingElement) {
                        var astProps = child.openingElement.attributes;
                        var len = astProps.length;
                        while (parentCanBeKeyed === false && len-- > 0) {
                            var prop = astProps[len];
                            if (prop.name && prop.name.name === "key") {
                                parentCanBeKeyed = true;
                            }
                        }
                    }
                }
            }
            // Fix: When there is single child parent cant be keyed either, its faster to use patch than patchKeyed routine in that case
            var hasSingleChild = children.length === 1;
            return {
                parentCanBeKeyed: hasSingleChild === false && parentCanBeKeyed,
                children: hasSingleChild ? children[0] : ts.createArrayLiteral(children)
            };
        }
        function createVNodeArgs(flags, type, className, children, props, key, ref, noNormalize, context) {
            var args = [];
            var hasClassName = !isNodeNull_1.default(className);
            var hasChildren = !isNodeNull_1.default(children);
            var hasProps = props.length ? true : false;
            var hasKey = !isNodeNull_1.default(key);
            var hasRef = !isNodeNull_1.default(ref);
            args.push(ts.createNumericLiteral(flags + ""));
            args.push(type);
            if (hasClassName) {
                args.push(className);
            }
            else if (hasChildren || hasProps || hasKey || hasRef || noNormalize) {
                args.push(ts.createNull());
            }
            if (hasChildren) {
                args.push(children);
            }
            else if (hasProps || hasKey || hasRef || noNormalize) {
                args.push(ts.createNull());
            }
            if (hasProps) {
                props.length === 1
                    ? args.push(props[0])
                    : args.push(createAssignHelper_1.default(context, props));
            }
            else if (hasKey || hasRef || noNormalize) {
                args.push(ts.createNull());
            }
            if (hasKey) {
                args.push(key);
            }
            else if (hasRef || noNormalize) {
                args.push(ts.createNull());
            }
            if (hasRef) {
                args.push(ref);
            }
            else if (noNormalize) {
                args.push(ts.createNull());
            }
            if (noNormalize) {
                args.push(ts.createTrue());
            }
            return args;
        }
    };
}
exports.default = default_1;
